<h1 align="center">
  <br>
  <img src="https://upload.wikimedia.org/wikipedia/commons/4/44/Spring_Framework_Logo_2018.svg" alt="LaTeX Logo" width="200">
  <br>Spring Protected Area<br>
  <sub><em>Example Project</em></sub>
</h1>

This is a sample project implementing a Spring Boot application with Spring Security and JWT (JSON Web Token) for authentication. PostgreSQL is used as the database.

## Requirements

To run this project, you'll need the following:

- Java Development Kit (JDK) version 21
- Apache Maven
- An IDE of your choice (e.g., IntelliJ IDEA, Eclipse)
- PostgreSQL database

## Configuration

1. Clone the repository to your local machine:

```bash
git clone https://git.armin-wolf.de/generalbgh/spring-protected-area-example.git
```

2. Open the project in your IDE.

3. Configure the database connection in the `application.yaml` file located in the `src/main/resources` directory:

```yaml
spring:
  datasource:
    platform: postgres
    url: jdbc:postgresql://localhost:5432/CHANGE_ME
    driver-class-name: org.postgresql.Driver
    username: CHANGE_ME
    password: CHANGE_ME
  jpa:
    database-platform: org.hibernate.dialect.PostgreSQLDialect
    hibernate:
      ddl-auto: update
    show-sql: false
```

4. Start the application:

```bash
mvn spring-boot:run
```

The application should now be accessible at `http://localhost:8080`.

## Usage

Once the application is running, you can access various endpoints to retrieve or create resources. Here are some examples:

- `POST /login`: Authenticate using username and password. This endpoint returns a JWT token.
- `POST /register`: Register a new user with a username and password.

Note that for protected endpoints, you'll need to provide a valid JWT token in the Authorization header. You can do this by sending the token in the format `Bearer <TOKEN>`.

## Further Information

For more information on Spring Boot, Spring Security, and JWT, refer to the official documentation:

- [Spring Boot Documentation](https://docs.spring.io/spring-boot/docs/current/reference/htmlsingle/)
- [Spring Security Documentation](https://docs.spring.io/spring-security/site/docs/current/reference/html5/)
- [JWT.io](https://jwt.io/)

