package de.arminwolf;

import de.arminwolf.models.Role;
import de.arminwolf.models.User;
import de.arminwolf.services.RoleService;
import de.arminwolf.services.UserService;
import jakarta.transaction.Transactional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;

import java.util.Set;

@SpringBootApplication
public class Application {

	@Autowired
	UserService userService;

	@Autowired
	RoleService roleService;

	private static Logger LOG = LoggerFactory.getLogger(Application.class);


	public static void main(String[] args) {
		SpringApplication.run(Application.class, args);
	}


	@Bean
	@Transactional
	CommandLineRunner run() {
		return args -> {
			LOG.info("Creating default ADMIN Account.");

			User adminAccount = userService
					.createUser("Admin",
							"Admin",
							"admin",
							"admin",
							Set.of(roleService.getDefaultRole(), roleService.getAdminRole()));
			boolean existingAdminAccount = userService.existsByUsername(adminAccount.getUsername());
			if (!existingAdminAccount) {
				LOG.info("Admin Account created.");
				userService.save(adminAccount);
			} else {
				LOG.info("Admin Account already exists.");
			}
		};
	}
}
