package de.arminwolf.configs;

import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.AllArgsConstructor;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;
import java.util.Objects;

@AllArgsConstructor
public class JwtAuthFilter extends OncePerRequestFilter {

	public static final String BEARER_TOKEN_PREFIX      = "Bearer ";
	public static final String AUTHORIZATION            = "Authorization";
	public static final int    BEARER_TOKEN_START_INDEX = 7;

	protected final AuthenticationProvider authenticationProvider;


	@Override
	protected void doFilterInternal(final HttpServletRequest request,
									final HttpServletResponse response,
									final FilterChain filterChain) throws ServletException, IOException {

		String authorization = request.getHeader(AUTHORIZATION);
		if (Objects.nonNull(authorization) && authorization.startsWith(BEARER_TOKEN_PREFIX)) {
			String         token          = authorization.substring(BEARER_TOKEN_START_INDEX);
			Authentication authentication = authenticationProvider.validateToken(token);
			try {
				SecurityContextHolder.getContext().setAuthentication(authentication);
			} catch (RuntimeException e) {
				SecurityContextHolder.clearContext();
				throw e;
			}
		}

		filterChain.doFilter(request, response);
	}

}
