package de.arminwolf.configs;

import com.nimbusds.jose.jwk.JWKSet;
import com.nimbusds.jose.jwk.RSAKey;
import com.nimbusds.jose.jwk.source.ImmutableJWKSet;
import de.arminwolf.util.RSAKeyProvider;
import lombok.AllArgsConstructor;
import org.springframework.context.annotation.Bean;
import org.springframework.security.oauth2.jwt.JwtDecoder;
import org.springframework.security.oauth2.jwt.JwtEncoder;
import org.springframework.security.oauth2.jwt.NimbusJwtDecoder;
import org.springframework.security.oauth2.jwt.NimbusJwtEncoder;
import org.springframework.stereotype.Component;

@AllArgsConstructor
@Component
public class TokenConfig {

	private final RSAKeyProvider keyProvider;

	@Bean
	public JwtDecoder jwtDecoder() {
		return NimbusJwtDecoder.withPublicKey(keyProvider.getPublicKey()).build();
	}

	@Bean
	public JwtEncoder jwtEncoder() {
		RSAKey rsaKey = new RSAKey.Builder(keyProvider.getPublicKey()).privateKey(keyProvider.getPrivateKey()).build();
		return new NimbusJwtEncoder(new ImmutableJWKSet<>(new JWKSet(rsaKey)));
	}

}
