package de.arminwolf.services;

import de.arminwolf.configs.AuthenticationProvider;
import de.arminwolf.exceptions.AppException;
import de.arminwolf.models.User;
import de.arminwolf.models.dto.LoginRequestDTO;
import de.arminwolf.models.dto.RegistrationRequestDTO;
import de.arminwolf.models.dto.UserDTO;
import de.arminwolf.models.mapper.RegistrationMapper;
import de.arminwolf.models.mapper.UserMapper;
import de.arminwolf.repositories.RoleRepository;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

@AllArgsConstructor
@Service
public class AuthenticationService {

	private final UserService                userService;

	private final AuthenticationProvider authenticationProvider;
	private final PasswordEncoder        passwordEncoder;
	private final RoleRepository             roleRepository;


	@Autowired
	private RegistrationMapper registrationMapper;

	@Autowired
	private UserMapper userMapper;

	public UserDTO login(LoginRequestDTO loginRequestDTO) {
		UserDTO userDTO = userService.login(loginRequestDTO.getUsername(), loginRequestDTO.getPassword());
		userDTO.setToken(authenticationProvider.createToken(userDTO));
		return userDTO;
	}


	public UserDTO register(final RegistrationRequestDTO registerRequestDTO) {
		boolean userExists = userService.existsByUsername(registerRequestDTO.getUsername());
		if (userExists) {
			throw new AppException("User already exists", HttpStatus.BAD_REQUEST);
		}

		// save user
		User user = registrationMapper.registrationRequestDTOToUser(registerRequestDTO);
		user.setRoles(userService.getDefaultRoles());
		userService.save(user);

		// convert the latest user to a userDTO
		final UserDTO userDTO = userMapper.userToUserDTO(user);
		userDTO.setToken(authenticationProvider.createToken(userDTO));
		return userDTO;
	}
}
