package de.arminwolf.services;

import de.arminwolf.exceptions.AppException;
import de.arminwolf.models.Role;
import de.arminwolf.models.User;
import de.arminwolf.models.dto.RoleDTO;
import de.arminwolf.models.dto.UserDTO;
import de.arminwolf.models.mapper.RoleMapper;
import de.arminwolf.models.mapper.UserMapper;
import de.arminwolf.repositories.RoleRepository;
import de.arminwolf.repositories.UserRepository;
import jakarta.transaction.Transactional;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.nio.CharBuffer;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
public class UserService {

	private final UserRepository userRepository;
	private final RoleRepository roleRepository;
	private final PasswordEncoder passwordEncoder;

	@Autowired
	private UserMapper userMapper;

	@Autowired
	private RoleMapper roleMapper;




	public User createUser(final String firstName, final String lastName, final String username, final String password, final Set<Role> allRoles) {
		return User.builder().id(0L).email("")
				.firstName(firstName)
				.lastName(lastName)
				.username(username)
				.password(passwordEncoder.encode(CharBuffer.wrap(password)))
				.roles(allRoles).build();
	}


	public boolean existsByUsername(final String username) {
		return userRepository.existsByUsername(username);
	}


	public Optional<User> findByUsername(final String username) {
		return userRepository.findByUsername(username);
	}


	public Set<Role> getDefaultRoles() {
		Role user = roleRepository.findByName("USER");
		if (Objects.nonNull(user)) {
			return Set.of(user);
		} else {
			Role role = Role.builder().name("USER").build();
			return Set.of(roleRepository.save(role));
		}
	}


	public UserDTO login(final String username, final String password) {
		User user = userRepository.findByUsername(username)
				.orElseThrow(() -> new AppException("Unknown user", HttpStatus.NOT_FOUND));

		if (passwordEncoder.matches(CharBuffer.wrap(password), user.getPassword())) {
			/*
			Set<Role> roles = user.getRoles();
			Set<RoleDTO> roleDTO = roles.stream()
					.map(role -> new RoleDTO(role.getId(), role.getName()))
					.collect(Collectors.toSet());

			return new UserDTO(user.getId(),
					user.getFirstName(),
					user.getLastName(),
					user.getUsername(),
					null, roleDTO);
			 */
			return userMapper.userToUserDTO(user);

		}

		throw new AppException("Invalid password", HttpStatus.BAD_REQUEST);
	}


	@Transactional
	public User save(final User createdUser) {
		return userRepository.save(createdUser);
	}
}
